import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'dart:math';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:linkv_rtm/rtm_flutter_plugin.dart';
import 'package:linkv_rtm/linkv_video_widget.dart';
import 'package:linkv_rtm/linkv_api_defines.dart';
import 'package:linkv_rtm/linkv_video_callback.dart';

class LivePage extends StatefulWidget {
  final bool isMaster;
  final String roomId;
  final String userId;

  LivePage(this.isMaster, this.roomId, this.userId);

  @override
  State<StatefulWidget> createState() {
    return new _LivePageState();
  }
}

class _LivePageState extends State<LivePage> implements LinkVVideoCallback {
  var TAG = "livepage";
  String _remoteUid = "";
  bool _position = true;

  int _local_width = 0;
  int _local_height = 0;
  int _local_fps = 0;

  int _remote_width = 0;
  int _remote_height = 0;
  int _remote_fps = 0;

  final _msgFieldController = TextEditingController();

  // 消息展示框
  final _msgListController = TextEditingController();

  @override
  void initState() {
    super.initState();

    loginRoom();
    RtmFlutterPlugin.setVideoCallback(this);

    RtmFlutterPlugin.setBeautyLevel(LVBeauty.Beauty, 0.8);
    RtmFlutterPlugin.setBeautyLevel(LVBeauty.Bright, 0.8);
    RtmFlutterPlugin.setBeautyLevel(LVBeauty.Tone, 0.5);
  }

  void loginRoom() async {
    RtmFlutterPlugin.loginRoom(
        widget.roomId, widget.userId, widget.isMaster, false);
  }

  void addRemoteView(String streamId) {
    if (streamId == null || streamId.length == 0 || streamId == _remoteUid)
      return;

    print(TAG + "addRemoteView streamId = " + streamId);
    setState(() {
      _remoteUid = streamId;
    });
    RtmFlutterPlugin.startPlayingStream(streamId, true);
//    RtmFlutterPlugin.setPlayQualityMonitor(2);
  }

  @override
  void dispose() {
    print(TAG + "dispose");
    if (widget.isMaster) {
      print(TAG + "dispose  stopPreview");
      RtmFlutterPlugin.stopCapture();
      RtmFlutterPlugin.stopPublishStream();
    }
    if (_remoteUid.length != 0) {
      RtmFlutterPlugin.stopPlayingStream(_remoteUid);
    }

    RtmFlutterPlugin.logoutRoom();

    super.dispose();
  }

  void onClickSend() {
    RtmFlutterPlugin.sendRoomMessage(widget.roomId, _msgFieldController.text, null)
        .then((lvimMsg) {
      String msg = lvimMsg.eCode == 0 ? "发送消息成功" : "发送消息失败";
      Fluttertoast.showToast(msg: msg);
    });

//    onClickToChangeBeauty();
  }

  double bright = 0;
  // 设置美颜参数.
  void onClickToChangeBeauty() {
    // 每次增加0.2
    bright = bright + 0.2;
    if(bright >1){
      bright = 0;
    }
    RtmFlutterPlugin.setBeautyLevel(LVBeauty.Tone, 1-bright);
    RtmFlutterPlugin.setBeautyLevel(LVBeauty.Bright, bright);
    RtmFlutterPlugin.setBeautyLevel(LVBeauty.Beauty, bright);
  }

  void onClickSwitchCamera() {
    _position = !_position;
//    RtmFlutterPlugin.switchCamera(_position);
    if (_position) {
      RtmFlutterPlugin.startCapture();
    }else{
      RtmFlutterPlugin.stopCapture();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(title: Text("room_id: ${widget.roomId}")),
        body: Container(
          color: Colors.black26,
          width: MediaQuery.of(context).size.width,
          child: Column(
            children: <Widget>[
              msgContainer(),
              buildLocalView(),
              buildRemoteView(),
              switchCamera(context),
              msgList(),
            ],
          ),
        ));
  }

  Widget msgContainer() {
    return Container(
        margin: EdgeInsets.only(top: 10, left: 20, right: 20),
//        width: MediaQuery.of(context).size.width - 40,
        height: 90,
        child: Row(
          children: <Widget>[msgInput(), sendBtn()],
        ));
  }

  Widget msgInput() {
    return Container(
        margin: EdgeInsets.only(top: 0, left: 0, right: 20),
        width: MediaQuery.of(context).size.width * 0.6 - 10,
        height: 50,
        color: Colors.cyan,
        child: TextField(
          controller: _msgFieldController,
          decoration: InputDecoration(
              contentPadding:
                  const EdgeInsets.only(left: 10, top: 12, bottom: 12),
              hintText: "Please enter msg",
              enabledBorder: OutlineInputBorder(
                  borderSide: BorderSide(color: Colors.grey)),
              focusedBorder: OutlineInputBorder(
                  borderSide: BorderSide(color: Colors.cyan))),
        ));
  }

  Widget sendBtn() {
    return Container(
        margin: EdgeInsets.only(top: 0, left: 0, right: 0),
        height: 50,
        color: Colors.cyan,
        child: CupertinoButton(
          child: Text(
            "send",
            style: TextStyle(color: Colors.white),
          ),
          onPressed: onClickSend,
        ));
  }

  Widget switchCamera(BuildContext context) {
    return Container(
      margin: EdgeInsets.only(top: 10, left: 20, right: 20),
      width: MediaQuery.of(context).size.width - 40,
      height: 50,
      color: Colors.cyan,
      child: CupertinoButton(
        child: Text(
          "Switch Camera",
          style: TextStyle(color: Colors.white),
        ),
        onPressed: onClickSwitchCamera,
      ),
    );
  }

  // 消息展示
  Widget msgList() {
    return Container(
        margin: EdgeInsets.only(top: 20, left: 10, right: 10),
        width: MediaQuery.of(context).size.width - 20,
        height: 50,
        color: Colors.cyan,
        child: TextField(
          controller: _msgListController,
          decoration: InputDecoration(
              contentPadding:
                  const EdgeInsets.only(left: 10, top: 10, bottom: 12),
              hintText: "Please enter msg",
              enabledBorder: OutlineInputBorder(
                  borderSide: BorderSide(color: Colors.grey)),
              focusedBorder: OutlineInputBorder(
                  borderSide: BorderSide(color: Colors.cyan))),
        ));
  }

  Widget buildRemoteView() {
    if (_remoteUid.length == 0) return Container();

    return Column(
      children: <Widget>[
        Padding(
          padding: EdgeInsets.only(top: 10),
        ),
        Stack(
          children: <Widget>[
            Container(
              width: MediaQuery.of(context).size.width,
              height: MediaQuery.of(context).size.height * 0.3 - 80,
              child: LinkVVideoWidget(uid: _remoteUid, isLocalPreview: false),
            ),
            remoteFpsView(),
          ],
        )
      ],
    );
  }

  Widget buildLocalView() {
    if (!widget.isMaster) return Container();
    return Column(
      children: <Widget>[
        Container(
          padding: const EdgeInsets.symmetric(vertical: 10),
          child: Text("current uid: ${widget.userId}"),
        ),
        Stack(
          children: <Widget>[
            Container(
              color: Colors.cyan,
              width: MediaQuery.of(context).size.width,
              height: MediaQuery.of(context).size.height * 0.3 - 80,
              child: LinkVVideoWidget(uid: widget.userId, isLocalPreview: true),
            ),
            localFpsView(),
          ],
        )
      ],
    );
  }

  Widget localFpsView() {
    if (_local_width == 0) return Container();
    return Positioned(
      left: 20,
      top: 10,
      child: Text(
          "${_local_width}x$_local_height / ${_local_fps}fps \n${widget.userId}"),
    );
  }

  Widget remoteFpsView() {
    if (_remote_width == 0) return Container();
    return Positioned(
      left: 20,
      top: 10,
      child: Text(
          "${_remote_width}x$_remote_height / ${_remote_fps}fps \n$_remoteUid"),
    );
  }

  // MARK: - LinkVVideoCallback
  @override
  void onAddRemoter(String userId) {
    addRemoteView(userId);
  }

  @override
  void onDeleteRemoter(String streamId) {
    print(TAG + " onRemoteStreamEnd streamId = $streamId");
    if (streamId != null && streamId == _remoteUid) {
      setState(() {
        _remoteUid = "";
      });
      RtmFlutterPlugin.stopPlayingStream(streamId);
    }
  }

  @override
  void onRoomConnected() {
    print(TAG + " onRoomConnected");
    if (widget.isMaster) {
      // 开始推流
      RtmFlutterPlugin.startPublishing();
    }
    Fluttertoast.showToast(
        msg: "登陆成功",
        toastLength: Toast.LENGTH_SHORT,
        gravity: ToastGravity.CENTER);
  }

  @override
  void onPlayQualityUpdate(LVVideoStatistic lvVideoStatistic, String s) {
    print(TAG +
        " onPlayQualityUpdate streamId = $s fps = ${lvVideoStatistic.videoFps} width = ${lvVideoStatistic.frameWidth}");
    setState(() {
      _remote_fps = lvVideoStatistic.videoFps;
      _remote_width = lvVideoStatistic.frameWidth;
      _remote_height = lvVideoStatistic.frameHeight;
    });
  }

  @override
  void onPlayStateUpdate(int state, String streamId) {
    print(TAG + "Wing onPlayStateUpdate streamId = $streamId state = $state");
  }

  @override
  void onPublishQualityUpdate(LVVideoStatistic lvVideoStatistic) {
    print(TAG +
        "onPublishQualityUpdate fps = ${lvVideoStatistic.videoFps} width = ${lvVideoStatistic.frameWidth}");
    setState(() {
      _local_fps = lvVideoStatistic.videoFps;
      _local_width = lvVideoStatistic.frameWidth;
      _local_height = lvVideoStatistic.frameHeight;
    });
  }

  @override
  void onPublishStateUpdate(int state) {
    print(TAG + "Wing onPublishStateUpdate state = $state");
  }

  @override
  void onRoomDisconnect(int code) {
    print(TAG + " onRoomDisconnect");
  }

  @override
  void onRoomReconnect() {
    print(TAG + " onRoomReconnect");
  }

  @override
  void onVideoSizeChanged(String streamID, int width, int height) {
    print(TAG +
        " onVideoSizeChanged  streamID " +
        streamID +
        "  width " +
        width.toString());
  }

  @override
  void onRoomMessageReceive(LVIMMsg msg) {
    var msgContent = msg.msgContent.toString();
    print(TAG + " onRoomMessageReceive" + msg.toString());
    _msgListController.text = msg.fromID + " : " + msgContent;
  }

  @override
  void onUserEnableVideo(String userId, bool enable) {
    print(TAG + " onUserEnableVideo userId = " + userId +"   enable = " + enable.toString());
  }
}
