#import "RtmFlutterPlugin.h"
#if __has_include(<rtm_flutter_plugin/rtm_flutter_plugin-Swift.h>)
#import <rtm_flutter_plugin/rtm_flutter_plugin-Swift.h>
#else
// Support project import fallback if the generated compatibility header
// is not copied when this plugin is created as a library.
// https://forums.swift.org/t/swift-static-libraries-dont-copy-generated-objective-c-header/19816
//#import "rtm_flutter_plugin-Swift.h"
#endif

#import "LVRTMEngine.h"
#import "FlutterSurfaceViewFactory.h"

@interface RtmFlutterPlugin ()<FlutterPlugin, RoomCallbackProtocl, LVIMModuleEventDelegate, LVIMReceiveMessageDelegate>

@property (nonatomic, strong) FlutterMethodChannel *channel;
@property (nonatomic, strong) LVRTMEngine *engine;

@property (nonatomic, strong) FlutterResult initResult;
@property (nonatomic, copy) NSString *uid;
@property (nonatomic, assign) BOOL isRequestIMToken;

@end

@implementation RtmFlutterPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    
    FlutterMethodChannel *channel = [FlutterMethodChannel methodChannelWithName:@"rtm_flutter_plugin" binaryMessenger:registrar.messenger];
    RtmFlutterPlugin *plugin = [RtmFlutterPlugin new];
    plugin.channel = channel;
    [registrar addMethodCallDelegate:plugin channel:channel];
    
    // 注册我们创建的view，这里的withId需要和flutter侧相同
    [registrar registerViewFactory:[FlutterSurfaceViewFactory new] withId:@"surface"];
}

- (void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result {
    if ([call.method isEqualToString:@"getPlatformVersion"]) {
        result([UIDevice currentDevice].systemVersion);
    } else if ([call.method isEqualToString:@"initSDK"]) {
        [self initSDK:call result:result];
    }  else if ([call.method isEqualToString:@"loginUser"]) {
        [self loginUser:call result:result];
    } else if ([call.method isEqualToString:@"setLogOpen"]) {
        [self setLogOpen:call result:result];
    }  else if ([call.method isEqualToString:@"setVideoConfig"]) {
        [self setVideoConfig:call result:result];
    }  else if ([call.method isEqualToString:@"getSdkVersion"]) {
        [self getSdkVersion:call result:result];
    }  else if ([call.method isEqualToString:@"sendPrivateMessage"]) {
        [self sendPrivateMessage:call result:result];
    } else if ([call.method isEqualToString:@"sendRoomMessage"]) {
        [self sendRoomMessage:call result:result];
    }  else if ([call.method isEqualToString:@"unInitSDK"]) {
        [self unInitSDK:call result:result];
    }  else if ([call.method isEqualToString:@"loginRoom"]) {
        [self loginRoom:call result:result];
    }  else if ([call.method isEqualToString:@"setPublishQualityMonitorCycle"]) {
        [self setPublishQualityMonitorCycle:call result:result];
    }  else if ([call.method isEqualToString:@"startPublishing"]) {
        [self startPublishing:call result:result];
    }  else if ([call.method isEqualToString:@"setPlayQualityMonitor"]) {
        [self setPlayQualityMonitor:call result:result];
    }  else if ([call.method isEqualToString:@"startPlayingStream"]) {
        [self startPlayingStream:call result:result];
    } else if ([call.method isEqualToString:@"stopPlayingStream"]) {
        [self stopPlayingStream:call result:result];
    }  else if ([call.method isEqualToString:@"logoutRoom"]){
        [self logoutRoom:call result:result];
    }  else if ([call.method isEqualToString:@"logoutUser"]){
             [self logoutUser:call result:result];
    }  else if ([call.method isEqualToString:@"stopPreview"]) {
        [self stopPreview:call result:result];
    }  else if ([call.method isEqualToString:@"stopPublishStream"]) {
        [self stopPublishStream:call result:result];
    }  else if ([call.method isEqualToString:@"switchCamera"]) {
        [self switchCamera:call result:result];
    }  else if ([call.method isEqualToString:@"setMuteOutput"]) {
        [self setMuteOutput:call result:result];
    }  else if ([call.method isEqualToString:@"isIMAuthed"]) {
        [self isIMAuthed:call result:result];
    } else if ([call.method isEqualToString:@"isIMLoginSucceed"]) {
        [self isIMLoginSucceed:call result:result];
    } else if ([call.method isEqualToString:@"setBeautyLevel"]) {
        [self setBeautyLevel:call result:result];
    } else if ([call.method isEqualToString:@"setIMToken"]) {
        [self setIMToken:call result:result];
    } else if ([call.method isEqualToString:@"requestDebugToken"]) {
        [self requestDebugToken:call result:result];
    } else if ([call.method isEqualToString:@"startCapture"]) {
        [self startCapture];
    } else if ([call.method isEqualToString:@"stopCapture"]) {
        [self stopCapture];
    } else if ([call.method isEqualToString:@"sendEventMessage"]) {
        [self sendEventMessage:call result:result];
    } else if ([call.method isEqualToString:@"deleteLocalPrivateSession"]) {
        [self deleteLocalPrivateSession:call result:result];
    } else if ([call.method isEqualToString:@"clearPrivateSessionUnreadMsg"]) {
        [self clearPrivateAllUnreadMsg:call result:result];
    } else if ([call.method isEqualToString:@"clearPrivateAllUnreadMsg"]) {
        [self clearPrivateAllUnreadMsg:call result:result];
    } else if ([call.method isEqualToString:@"updatePrivateMsgAsReadByStime"]) {
        [self updatePrivateMsgAsReadByStime:call result:result];
    } else if ([call.method isEqualToString:@"queryLocalPrivateHistoryMessage"]) {
        [self queryLocalPrivateHistoryMessage:call result:result];
    } else if ([call.method isEqualToString:@"querySessionList"]) {
        [self querySessionList:call result:result];
    } else if ([call.method isEqualToString:@"sendMessage"]) {
        [self sendMessage:call result:result];
    } else if ([call.method isEqualToString:@"setPrivateDBStorageMax"]) {
        [self setPrivateDBStorageMax:call result:result];
    } else if ([call.method isEqualToString:@"queryUnReadMessageNum"]) {
        [self queryUnReadMessageNum:call result:result];
    }
}

#pragma mark - Flutter Call Native
- (void)initSDK:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    self.initResult = result;
    
    NSString *rtcAppID = dict[@"rtcAppID"];
    NSString *rtcAppKey = dict[@"rtcAppKey"];
    NSString *imAppID = dict[@"imAppID"];
    NSString *imAppKey = dict[@"imAppKey"];
    int sdkType = [dict[@"sdkType"] intValue];
    BOOL isDebugEnv = [dict[@"isDebugEnv"] boolValue];
    BOOL isInternationalEnv = [dict[@"isInternationalEnv"] boolValue];
    
    self.engine = [LVRTMEngine createEngine:sdkType rtcAppId:rtcAppID rtcAppKey:rtcAppKey imAppId:imAppID imAppKey:imAppKey isTestEnv:isDebugEnv isInternationalEnv:isInternationalEnv delegate:self];
}

- (void)loginUser:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    self.uid = dict[@"userId"];
    int rs = [self.engine loginIM:dict[@"userId"]];
    result(@(rs));
    
    NSLog(@"[Wing] loginIM uid = %@", self.uid);
}

- (void)setLogOpen:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine setLogOpen:[dict[@"isOpen"] boolValue]];
    result(@(YES));
}

- (void)setVideoConfig:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine setVideoConfig:[dict[@"videoConfigLevel"] integerValue]];
    result(@(YES));
}

- (void)getSdkVersion:(FlutterMethodCall *)call result:(FlutterResult)result {
    result([self.engine sdkVersion]);
}

- (void)sendPrivateMessage:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    int sub = [dict[@"sub"] intValue];
    NSString *tid = dict[@"tid"];
    NSString *type = dict[@"type"];
    NSString *content = dict[@"content"];
    
    [self.engine sendPrivateMessage:sub tid:tid type:type content:content complete:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, LVIMMessage *msg) {
        
        NSMutableDictionary *dictM = [self msgToDict:msg];
        dictM[@"eCode"] = @(ecode);
        result(dictM);
    }];
}

- (void)sendRoomMessage:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine sendRoomMessage:dict[@"targetId"] content:dict[@"msgContent"] msgType:dict[@"msgType"] complete:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, LVIMMessage *msg) {
        
        NSMutableDictionary *dictM = [self msgToDict:msg];
        dictM[@"eCode"] = @(ecode);
        result(dictM);
    }];
}

- (void)unInitSDK:(FlutterMethodCall *)call result:(FlutterResult)result {
    [self.engine unInitSDK];
    result(@(YES));
}

- (void)loginRoom:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine loginRoom:dict[@"userId"] roomId:dict[@"roomId"] isHost:[dict[@"isHost"] boolValue] isAudioOnly:[dict[@"isAudioOnly"] boolValue]];
    result(@(YES));
}

- (void)setPublishQualityMonitorCycle:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine setPublishQualityMonitorCycle:[dict[@"seconds"] intValue]];
    result(@(YES));
}

- (void)startPublishing:(FlutterMethodCall *)call result:(FlutterResult)result {
    [self.engine startPublishing];
    result(@(YES));
}

- (void)setPlayQualityMonitor:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine setPlayQualityMonitorCycle:[dict[@"seconds"] intValue]];
    result(@(YES));
}

- (void)startPlayingStream:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine startPlayingStream:dict[@"streamId"] inView:nil];
    result(@(YES));
}

- (void)stopPlayingStream:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine stopPlayingStream:dict[@"streamId"]];
    result(@(YES));
}

- (void)logoutRoom:(FlutterMethodCall *)call result:(FlutterResult)result {
    [self.engine logoutRoom];
    result(@(YES));
}


- (void)logoutUser:(FlutterMethodCall *)call result:(FlutterResult)result {
    [[LVIMSDK sharedInstance] logout];
    result(@(YES));
}


- (void)stopPreview:(FlutterMethodCall *)call result:(FlutterResult)result {
    
}

- (void)stopPublishStream:(FlutterMethodCall *)call result:(FlutterResult)result {
    [self.engine stopPublishStream];
    result(@(YES));
}

- (void)switchCamera:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine useFrontCamera:[dict[@"isFrontCamera"] boolValue]];
    result(@(YES));
}

- (void)setMuteOutput:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine enableMic:[dict[@"isMute"] boolValue]];
    result(@(YES));
}

- (void)isIMAuthed:(FlutterMethodCall *)call result:(FlutterResult)result {
    result(@([[LVIMSDK sharedInstance] isAuthed]));
}

- (void)isIMLoginSucceed:(FlutterMethodCall *)call result:(FlutterResult)result {
    result(@([[LVIMSDK sharedInstance] isAppUserLoginSucceed]));
}

- (void)setBeautyLevel:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    int beauty = [dict[@"beauty"] intValue];
    float level = [dict[@"level"] floatValue];
    
    switch (beauty) {
        case 0:
            [self.engine setBeautyLevel:level];
            break;
        case 1:
            [self.engine setBrightLevel:level];
            break;
        case 2:
            [self.engine setToneLevel:level];
            break;
        default:
            break;
    }
    
    result(@(YES));
}

- (void)setIMToken:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [[LVIMSDK sharedInstance] setIMToken:dict[@"userId"] token:dict[@"imToken"]];
    result(@(YES));
}

- (void)requestDebugToken:(FlutterMethodCall *)call result:(FlutterResult)result {
    
    [[LVIMSDK sharedInstance] requestDebugToken:^(NSInteger errorCode, NSDictionary *responseDict) {
        NSString *token = @"";
        if ([responseDict isKindOfClass:[NSDictionary class]]) {
            token = [NSString stringWithFormat:@"%@", responseDict[@"data"]];
        }
        result(@{@"code": @(errorCode), @"result": token});
    }];
}

- (void)startCapture {
    [self.engine startCapture];
}

- (void)stopCapture {
    [self.engine stopCapture];
}

- (void)sendEventMessage:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    NSString *tid = dict[@"tid"];
    NSString *type = dict[@"type"];
    NSString *content = dict[@"content"];
    
    LVIMMessage *msg = [LVIMMessage new];
    [msg buildEventRequest:self.uid tid:tid type:type content:content pushTitle:nil pushContent:nil targetAppID:nil targetAppUID:nil];
    [[LVIMSDK sharedInstance] sendMessage:msg context:nil callback:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, LVIMMessage *msg) {
        NSMutableDictionary *dictM = [self msgToDict:msg];
        dictM[@"eCode"] = @(ecode);
        result(dictM);
    }];
}

- (void)deleteLocalPrivateSession:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    NSString *uid = dict[@"uid"];
    int num = [[LVIMSDK sharedInstance] deleteLocalPrivateHistoryMessage:uid];
    result(@(num));
}

- (void)clearPrivateSessionUnreadMsg:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    NSString *uid = dict[@"uid"];
    int num = [[LVIMSDK sharedInstance] clearPrivateSessionUnreadMsg:uid];
    result(@(num));
}

- (void)clearPrivateAllUnreadMsg:(FlutterMethodCall *)call result:(FlutterResult)result {
    int num = [[LVIMSDK sharedInstance] clearPrivateAllUnreadMsg];
    result(@(num));
}

- (void)updatePrivateMsgAsReadByStime:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    NSString *uid = dict[@"uid"];
    NSInteger stime = [dict[@"stime"] integerValue];
    int num = [[LVIMSDK sharedInstance] updatePrivateMsgAsReadByStime:uid stime:stime];
    result(@(num));
}

- (void)queryLocalPrivateHistoryMessage:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    NSString *uid = dict[@"uid"];
    NSInteger dbid = [dict[@"dbid"] integerValue];
    int limit = [dict[@"limit"] intValue];
    dbid = dbid == 0 ? NSIntegerMax : dbid;
    
    NSMutableArray<LVIMMessage *> *arrM = [NSMutableArray arrayWithCapacity:limit];
    int num = [[LVIMSDK sharedInstance] queryLocalPrivateHistoryMessage:uid dbid:dbid limit:limit desc:YES rmsgs:arrM];
    
    NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
    dictM[@"result"] = @(num);
    if (num <= 0) {
        result(dictM);
        return;
    }
    
    NSMutableArray *msgs = [NSMutableArray arrayWithCapacity:num];
    for (LVIMMessage *msg in arrM) {
        [msgs addObject:[self msgToDict:msg]];
    }
    dictM[@"msgs"] = msgs;
    result(dictM);
}

- (void)querySessionList:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSMutableArray<LVIMSession *> *arrM = [NSMutableArray array];
    int num = [[LVIMSDK sharedInstance] querySessionList:arrM];
    
    NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
    dictM[@"result"] = @(num);
    
    if (num <= 0) {
        result(dictM);
        return;
    }
    
    NSMutableArray *msgs = [NSMutableArray arrayWithCapacity:num];
    for (LVIMSession *session in arrM) {
        NSMutableDictionary *sessionDict = [self msgToDict:session.lastMsg];
        sessionDict[@"unreadCount"] = @(session.unreadCount);
        [msgs addObject:sessionDict];
    }
    dictM[@"sessions"] = msgs;
    result(dictM);
}

- (void)sendMessage:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    int64_t cmsgid = [dict[@"cmsgid"] longLongValue];
    int sub = [dict[@"sub"] intValue];
    NSString *tid = dict[@"tid"];
    NSString *type = dict[@"type"];
    NSString *content = dict[@"content"];
    
    LVIMMessage *msg = [LVIMMessage new];
    [msg buildPrivateRequest:sub fid:self.uid tid:tid type:type content:content pushTitle:nil pushContent:nil extend3:nil extend4:nil targetAppID:nil targetAppUID:nil];
    msg.mLVsgID = cmsgid;
    
    [[LVIMSDK sharedInstance] sendMessage:msg context:nil callback:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, LVIMMessage *msg) {
        NSMutableDictionary *dictM = [self msgToDict:msg];
        dictM[@"eCode"] = @(ecode);
        result(dictM);
    }];
}

- (void)setPrivateDBStorageMax:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    int64_t max = [dict[@"max"] longLongValue];
    if (max <= 0) {
        result(@(-1));
        return;
    }
    
    int rs = [[LVIMSDK sharedInstance] setPrivateDBStorageMax:max];
    result(@(rs));
}

- (void)queryUnReadMessageNum:(FlutterMethodCall *)call result:(FlutterResult)result {
    int count = [[LVIMSDK sharedInstance] queryUnReadCount];
    result(@(count));
}

#pragma mark - RoomCallbackProtocl

- (void)onAddRemoterUser:(NSString *)uid {
    [self.channel invokeMethod:@"onRemoteStreamAdd" arguments:@{@"userId": uid ?: @""}];
}
- (void)onRemoteLeave:(NSString *)uid {
    [self.channel invokeMethod:@"onRemoteStreamEnd" arguments:@{@"userId": uid ?: @""}];
}

/**
 * 被踢出房间或断开房间链接
 * @param errorCode 错误码，参见Zego或LinkV错误码
 * @param roomId 房间id
 */
- (void)onRoomDisconnect:(int)errorCode roomId:(NSString *)roomId {
    NSLog(@"[Wing] onRoomDisconnect roomId = %@ errorCode = %d", roomId, errorCode);
    [self.channel invokeMethod:@"onRoomDisconnect" arguments:@{@"errorCode": @(errorCode), @"roomID": roomId ?: @""}];
}

- (void)onRoomConnected:(NSString *)roomId {
    NSLog(@"[Wing] onRoomConnected roomId = %@", roomId);
    [self.channel invokeMethod:@"onRoomConnected" arguments:@{@"roomID": roomId ?: @""}];
}
- (void)onVideoSizeChanged:(NSString *)uid width:(int)width height:(int)height {
    [self.channel invokeMethod:@"onVideoSizeChanged" arguments:@{@"streamID": uid, @"width": @(width), @"height": @(height)}];
}
- (void)onPublishStreamQualityUpdate:(NSString *)uid quality:(VideoQuality *)quality {
    [self.channel invokeMethod:@"onPublisherQualityUpdate" arguments:@{@"videoSentPackets": @(quality.videoSentPackets), @"videoSentKBytes": @(quality.videosentKbytes), @"videoFps": @(quality.videoFps), @"videoLostPackets": @(quality.videoLostPackets), @"videoBitrateBps": @(quality.videoBitratebps), @"videoRtt": @(quality.videoRtt), @"videoLostPercent": @(quality.videoLostPercent), @"frameWidth": @(quality.frameWidth), @"frameHeight": @(quality.frameHeight)}];
}
- (void)onRemoteQualityUpdate:(NSString *)uid quality:(VideoQuality *)quality {
    [self.channel invokeMethod:@"onRemoteQualityUpdate" arguments:@{@"streamID": uid, @"videoSentPackets": @(quality.videoSentPackets), @"videoSentKBytes": @(quality.videosentKbytes), @"videoFps": @(quality.videoFps), @"videoLostPackets": @(quality.videoLostPackets), @"videoBitrateBps": @(quality.videoBitratebps), @"videoRtt": @(quality.videoRtt), @"videoLostPercent": @(quality.videoLostPercent), @"frameWidth": @(quality.frameWidth), @"frameHeight": @(quality.frameHeight)}];
}

/**
 * 鉴权回调
 * @param errorCode 0表示鉴权成功，其余均为错误，参见Zego或LinkV错误码
 */
- (void)authComplete:(NSInteger)errorCode {
    if (!self.initResult) return;
    
    NSLog(@"[Wing] authComplete errorCode = %@", @(errorCode));
    self.initResult(@(errorCode));
    self.initResult = nil;
}

- (void)onPlayStateUpdate:(int)code userId:(nonnull NSString *)userId {
    [self.channel invokeMethod:@"onPlayStateUpdate" arguments:@{@"streamId": userId, @"state": @(code)}];
}
- (void)onPublishStateUpdate:(int)code {
    [self.channel invokeMethod:@"onPublishStateUpdate" arguments:@{@"state": @(code)}];
}

/// 收到房间消息
/// 如果返回非0, 则这条消息会在下次冷启动或再次检测到时继续调用本方法
- (int)onRoomMessageReceive:(LVIMMessage *)msg {
    [self.channel invokeMethod:@"onRoomMessageReceive" arguments:[self msgToDict:msg]];
    
    return 0;
}

#pragma mark - LVIMModuleEventDelegate
/**
 @brief 请求token的事件
 **/
-(void)onQueryIMToken {
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onQueryIMToken" arguments:nil];
    });
}

/**
 @brief 连接成功事件(暂不提供)
 @param context 状态信息
 **/
-(void)onIMConnected:(void*)context {
    NSLog(@"[Wing] onIMConnected context = %@", context);
}

/**
 @brief 连接断开事件(暂不提供)
 **/
-(void)onIMLosted {
    NSLog(@"[Wing] onIMLosted");
}

/**
 @brief 认证成功事件
 @param uid 用户ID
 @param token token (IM)
 @param unReadMsgSize 未读消息数量(粗略)
 **/
-(void)onIMAuthSuccessed:(NSString*)uid token:(NSString*)token unReadMsgSize:(int)unReadMsgSize {
    NSLog(@"[Wing] onIMAuthSuccessed uid = %@, token = %@", uid, token);
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onIMAuthSucceed" arguments:@{@"uid": uid ?: @"", @"token": token ?: @"", @"unReadMsgSize": @(unReadMsgSize)}];
    });
}

/**
 @brief 认证失败事件
 @param uid 用户ID
 @param token 认证码(im)
 @param ecode 错误码
 @param rcode 响应码
 @param expired 是否是过期了
 **/
-(void)onIMAuthFailed:(NSString*)uid token:(NSString*)token ecode:(int)ecode rcode:(int)rcode expired:(BOOL)expired {
    NSLog(@"[Wing] onIMAuthFailed uid = %@, token = %@, ecode = %@, rcode = %@, expired = %d", uid, token, @(ecode), @(rcode), expired);
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onIMAuthFailed" arguments:@{@"uid": uid ?: @"", @"token": token ?: @"", @"ecode": @(ecode), @"rcode": @(rcode), @"isTokenExpired": @(expired)}];
    });
}

/**
 @brief 认证过期
 @param uid 用户ID
 @param token 认证码(im)
 @param owner 所属者
 **/
-(void)onIMTokenExpired:(NSString*)uid token:(NSString*)token owner:(NSString*)owner {
    NSLog(@"[Wing] onIMTokenExpired uid = %@", uid);
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onIMTokenExpired" arguments:@{@"uid": uid ?: @"", @"token": token ?: @"", @"emsg": owner}];
    });
}

#pragma mark - LVIMReceiveMessageDelegate
/**
 @brief 消息过滤器
 @param diytype 消息类型的CRC32
 @param fromid 发送者ID
 @param toid 接收者ID
 @param msgtype 类型别名
 @param content 消息内容
 @param waitings 队列中等待处理的消息数量
 @param packetSize 当前数据包大小
 @param waitLength 等待处理的大小
 @param bufferSize 缓冲区最大大小
 @return 过滤丢弃掉这个消息返回YES, 继续处理返回NO
 **/
-(BOOL)onIMReceiveMessageFilter:(int32_t)diytype
                           fromid:(const char*)fromid
                             toid:(const char*)toid
                          msgtype:(const char*)msgtype
                          content:(const char*)content
                         waitings:(int)waitings
                       packetSize:(int)packetSize
                       waitLength:(int)waitLength
                     bufferSize:(int)bufferSize {
    
    return NO;
}

/**
 @brief 消息返回处理
 @param owner 这个消息属于哪个登陆用户
 @param msg 消息内容
 @param waitings 队列中等待处理的消息数量
 @param packetSize 当前数据包大小
 @param waitLength 等待处理的大小
 @param bufferSize 缓冲区最大大小
 @return 处理成功返回0, 否则返回非0
 @warning 如果返回非0, 则这条消息会在下次冷启动或再次检测到时继续调用本方法
 **/
-(int)onIMReceiveMessageHandler:(NSString*)owner
                          immsg:(LVIMMessage*)msg
                       waitings:(int)waitings
                     packetSize:(int)packetSize
                     waitLength:(int)waitLength
                     bufferSize:(int)bufferSize {

    [self.channel invokeMethod:@"onIMMessageReceive" arguments:[self msgToDict:msg]];
    
    return 0;
}

#pragma mark - private
- (NSDictionary *)removeNull:(NSDictionary *)dict {
    NSArray *allKeys = dict.allKeys;
    
    NSMutableDictionary *dictM = [dict mutableCopy];
    for (NSString *key in allKeys) {
        if ([dict[key] isKindOfClass:[NSNull class]]) {
            [dictM removeObjectForKey:key];
        }
    }
    
    return dictM;
}

- (NSMutableDictionary *)msgToDict:(LVIMMessage *)msg {
    NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
    if (!msg) return dictM;
    
    dictM[@"cmdType"] = @(msg.mCmdType);
    dictM[@"subType"] = @(msg.mSubType);
    dictM[@"fromID"] = msg.mFromID;
    dictM[@"toID"] = msg.mToID;
    dictM[@"msgType"] = msg.mExtend1;
    dictM[@"extend2"] = msg.mExtend2;
    dictM[@"extend3"] = msg.mExtend3;
    dictM[@"extend4"] = msg.mExtend4;
    dictM[@"extend5"] = msg.mExtend5;
    dictM[@"extend6"] = msg.mExtend6;
    dictM[@"pushTitle"] = msg.mPushTitle;
    dictM[@"cmsgid"] = @(msg.mLVsgID);
    dictM[@"smsgid"] = @(msg.mSMsgID);
    dictM[@"dbid"] = @(msg.mDBID);
    dictM[@"ctime"] = @(msg.mClientTimestamp);
    dictM[@"stime"] = @(msg.mServerTimestamp);
    dictM[@"origin"] = @(0);
    dictM[@"msgContent"] = msg.mContent;
    dictM[@"pushTitle"] = msg.mPushTitle;
    dictM[@"sendState"] = @(msg.send_state);
    
    return dictM;
}


@end
